@@ -107,7 +107,8 @@ void QgsStatisticalSummaryDockWidget::fieldChanged()
   if ( mFieldExpressionWidget->expression() != mExpression )
   {
     mExpression = mFieldExpressionWidget->expression();
-    mLastExpression.insert( mLayerComboBox->currentLayer()->id(), mFieldExpressionWidget->currentText() );
+    if ( QgsMapLayer *currentLayer = mLayerComboBox->currentLayer() )
+      mLastExpression.insert( currentLayer->id(), mFieldExpressionWidget->currentText() );
     refreshStatistics();
   }
 }
@@ -285,12 +286,15 @@ void QgsStatisticalSummaryDockWidget::updateNumericStatistics()
   const auto constVariantValues = variantValues;
   for ( const QVariant &value : constVariantValues )
   {
-    const double val = value.toDouble( &convertOk );
-    if ( convertOk )
-      values << val;
-    else if ( QgsVariantUtils::isNull( value ) )
-    {
+    if ( QgsVariantUtils::isNull( value ) )
       missingValues += 1;
+    else
+    {
+      const double val = value.toDouble( &convertOk );
+      if ( convertOk )
+        values << val;
+      else
+        missingValues += 1;
     }
   }
 
