@@ -222,6 +222,155 @@ void QgsPostgresProviderConnection::renameTablePrivate( const QString &schema, c
                            QgsPostgresConn::quotedIdentifier( newName ) ) );
 }
 
+QList<QgsAbstractDatabaseProviderConnection::TableProperty> QgsPostgresProviderConnection::tablesPrivate( const QString &schema, const QString &table, const TableFlags &flags, QgsFeedback *feedback ) const
+{
+  checkCapability( Capability::Tables );
+  QList<QgsPostgresProviderConnection::TableProperty> tables;
+  QString errCause;
+  // TODO: set flags from the connection if flags argument is 0
+  const QgsDataSourceUri dsUri { uri() };
+  QgsPostgresConn *conn = QgsPostgresConnPool::instance()->acquireConnection( dsUri.connectionInfo( false ), -1, false, feedback );
+  if ( feedback && feedback->isCanceled() )
+    return {};
+
+  if ( !conn )
+  {
+    errCause = QObject::tr( "Connection failed: %1" ).arg( uri() );
+  }
+  else
+  {
+    bool ok { false };
+    QVector<QgsPostgresLayerProperty> properties;
+    const bool aspatial { ! flags || flags.testFlag( TableFlag::Aspatial ) };
+    if ( ! table.isEmpty() )
+    {
+      QgsPostgresLayerProperty property;
+      ok = conn->supportedLayer( property, schema, table );
+      if ( ok )
+      {
+        properties.push_back( property );
+      }
+    }
+    else
+    {
+      ok = conn->supportedLayers( properties, false, schema == QStringLiteral( "public" ), aspatial, schema );
+    }
+
+    if ( ! ok )
+    {
+      if ( ! table.isEmpty() )
+      {
+        errCause = QObject::tr( "Could not retrieve table '%2' from %1" ).arg( uri(), table );
+      }
+      else
+      {
+        errCause = QObject::tr( "Could not retrieve tables: %1" ).arg( uri() );
+      }
+    }
+    else
+    {
+
+      bool dontResolveType = configuration().value( QStringLiteral( "dontResolveType" ), false ).toBool();
+      bool useEstimatedMetadata = configuration().value( QStringLiteral( "estimatedMetadata" ), false ).toBool();
+
+      // Cannot be const:
+      for ( auto &pr : properties )
+      {
+        // Classify
+        TableFlags prFlags;
+        if ( pr.relKind == Qgis::PostgresRelKind::View || pr.relKind == Qgis::PostgresRelKind::MaterializedView )
+        {
+          prFlags.setFlag( QgsPostgresProviderConnection::TableFlag::View );
+        }
+        if ( pr.relKind == Qgis::PostgresRelKind::MaterializedView )
+        {
+          prFlags.setFlag( QgsPostgresProviderConnection::TableFlag::MaterializedView );
+        }
+        if ( pr.relKind == Qgis::PostgresRelKind::ForeignTable )
+        {
+          prFlags.setFlag( QgsPostgresProviderConnection::TableFlag::Foreign );
+        }
+        if ( pr.isRaster )
+        {
+          prFlags.setFlag( QgsPostgresProviderConnection::TableFlag::Raster );
+        }
+        else if ( pr.nSpCols != 0 )
+        {
+          prFlags.setFlag( QgsPostgresProviderConnection::TableFlag::Vector );
+        }
+        else
+        {
+          prFlags.setFlag( QgsPostgresProviderConnection::TableFlag::Aspatial );
+        }
+        // Filter
+        if ( ! flags || ( prFlags & flags ) )
+        {
+          // retrieve layer types if needed
+          if ( ! dontResolveType && ( !pr.geometryColName.isNull() &&
+                                      ( pr.types.value( 0, Qgis::WkbType::Unknown ) == Qgis::WkbType::Unknown ||
+                                        pr.srids.value( 0, std::numeric_limits<int>::min() ) == std::numeric_limits<int>::min() ) ) )
+          {
+            conn->retrieveLayerTypes( pr, useEstimatedMetadata, feedback );
+          }
+          QgsPostgresProviderConnection::TableProperty property;
+          property.setFlags( prFlags );
+          for ( int i = 0; i < std::min( pr.types.size(), pr.srids.size() ) ; i++ )
+          {
+            property.addGeometryColumnType( pr.types.at( i ), QgsCoordinateReferenceSystem::fromEpsgId( pr.srids.at( i ) ) );
+          }
+          property.setTableName( pr.tableName );
+          property.setSchema( pr.schemaName );
+          property.setGeometryColumn( pr.geometryColName );
+          // These are candidates, not actual PKs
+          // property.setPrimaryKeyColumns( pr.pkCols );
+          property.setGeometryColumnCount( static_cast<int>( pr.nSpCols ) );
+          property.setComment( pr.tableComment );
+
+          // Get PKs
+          if ( pr.relKind == Qgis::PostgresRelKind::View
+               || pr.relKind == Qgis::PostgresRelKind::MaterializedView
+               || pr.relKind == Qgis::PostgresRelKind::ForeignTable )
+          {
+            // Set the candidates
+            property.setPrimaryKeyColumns( pr.pkCols );
+          }
+          else  // Fetch and set the real pks
+          {
+            try
+            {
+              const QList<QVariantList> pks = executeSqlPrivate( QStringLiteral( R"(
+              WITH pkrelid AS (
+              SELECT indexrelid AS idxri FROM pg_index WHERE indrelid=%1::regclass AND (indisprimary OR indisunique)
+                ORDER BY CASE WHEN indisprimary THEN 1 ELSE 2 END LIMIT 1)
+              SELECT attname FROM pg_index,pg_attribute, pkrelid
+              WHERE indexrelid=pkrelid.idxri AND indrelid=attrelid AND pg_attribute.attnum=any(pg_index.indkey);
+             )" ).arg( QgsPostgresConn::quotedValue( QString( QgsPostgresConn::quotedIdentifier( pr.schemaName ) + "." + QgsPostgresConn::quotedIdentifier( pr.tableName ) ) ) ), false );
+              QStringList pkNames;
+              for ( const QVariantList &pk : std::as_const( pks ) )
+              {
+                pkNames.push_back( pk.first().toString() );
+              }
+              property.setPrimaryKeyColumns( pkNames );
+            }
+            catch ( const QgsProviderConnectionException &ex )
+            {
+              QgsDebugError( QStringLiteral( "Error retrieving primary keys: %1" ).arg( ex.what() ) );
+            }
+          }
+
+          tables.push_back( property );
+        }
+      }
+    }
+    QgsPostgresConnPool::instance()->releaseConnection( conn );
+  }
+  if ( ! errCause.isEmpty() )
+  {
+    throw QgsProviderConnectionException( errCause );
+  }
+  return tables;
+}
+
 void QgsPostgresProviderConnection::renameVectorTable( const QString &schema, const QString &name, const QString &newName ) const
 {
   checkCapability( Capability::RenameVectorTable );
@@ -500,9 +649,16 @@ long long QgsPostgresProviderResultIterator::rowCountPrivate() const
 void QgsPostgresProviderConnection::vacuum( const QString &schema, const QString &name ) const
 {
   checkCapability( Capability::Vacuum );
-  executeSqlPrivate( QStringLiteral( "VACUUM FULL ANALYZE %1.%2" )
-                     .arg( QgsPostgresConn::quotedIdentifier( schema ),
-                           QgsPostgresConn::quotedIdentifier( name ) ), false );
+  if ( ! schema.isEmpty() && ! name.isEmpty() )
+  {
+    executeSqlPrivate( QStringLiteral( "VACUUM FULL ANALYZE %1.%2" )
+                       .arg( QgsPostgresConn::quotedIdentifier( schema ),
+                             QgsPostgresConn::quotedIdentifier( name ) ), false );
+  }
+  else
+  {
+    executeSqlPrivate( QStringLiteral( "VACUUM FULL ANALYZE" ) );
+  }
 }
 
 void QgsPostgresProviderConnection::createSpatialIndex( const QString &schema, const QString &name, const QgsAbstractDatabaseProviderConnection::SpatialIndexOptions &options ) const
@@ -601,131 +757,21 @@ void QgsPostgresProviderConnection::setFieldComment( const QString &fieldName, c
 
 QList<QgsPostgresProviderConnection::TableProperty> QgsPostgresProviderConnection::tables( const QString &schema, const TableFlags &flags, QgsFeedback *feedback ) const
 {
-  checkCapability( Capability::Tables );
-  QList<QgsPostgresProviderConnection::TableProperty> tables;
-  QString errCause;
-  // TODO: set flags from the connection if flags argument is 0
-  const QgsDataSourceUri dsUri { uri() };
-  QgsPostgresConn *conn = QgsPostgresConnPool::instance()->acquireConnection( dsUri.connectionInfo( false ), -1, false, feedback );
-  if ( feedback && feedback->isCanceled() )
-    return {};
+  return tablesPrivate( schema, QString(), flags, feedback );
+}
 
-  if ( !conn )
+QgsAbstractDatabaseProviderConnection::TableProperty QgsPostgresProviderConnection::table( const QString &schema, const QString &table, QgsFeedback *feedback ) const
+{
+  const QList<QgsPostgresProviderConnection::TableProperty> properties { tablesPrivate( schema, table, TableFlags(), feedback ) };
+  if ( ! properties.empty() )
   {
-    errCause = QObject::tr( "Connection failed: %1" ).arg( uri() );
+    return properties.first();
   }
   else
   {
-    QVector<QgsPostgresLayerProperty> properties;
-    const bool aspatial { ! flags || flags.testFlag( TableFlag::Aspatial ) };
-    bool ok = conn->supportedLayers( properties, false, schema == QStringLiteral( "public" ), aspatial, schema );
-    if ( ! ok )
-    {
-      errCause = QObject::tr( "Could not retrieve tables: %1" ).arg( uri() );
-    }
-    else
-    {
-
-      bool dontResolveType = configuration().value( QStringLiteral( "dontResolveType" ), false ).toBool();
-      bool useEstimatedMetadata = configuration().value( QStringLiteral( "estimatedMetadata" ), false ).toBool();
-
-      // Cannot be const:
-      for ( auto &pr : properties )
-      {
-        // Classify
-        TableFlags prFlags;
-        if ( pr.relKind == Qgis::PostgresRelKind::View || pr.relKind == Qgis::PostgresRelKind::MaterializedView )
-        {
-          prFlags.setFlag( QgsPostgresProviderConnection::TableFlag::View );
-        }
-        if ( pr.relKind == Qgis::PostgresRelKind::MaterializedView )
-        {
-          prFlags.setFlag( QgsPostgresProviderConnection::TableFlag::MaterializedView );
-        }
-        if ( pr.relKind == Qgis::PostgresRelKind::ForeignTable )
-        {
-          prFlags.setFlag( QgsPostgresProviderConnection::TableFlag::Foreign );
-        }
-        if ( pr.isRaster )
-        {
-          prFlags.setFlag( QgsPostgresProviderConnection::TableFlag::Raster );
-        }
-        else if ( pr.nSpCols != 0 )
-        {
-          prFlags.setFlag( QgsPostgresProviderConnection::TableFlag::Vector );
-        }
-        else
-        {
-          prFlags.setFlag( QgsPostgresProviderConnection::TableFlag::Aspatial );
-        }
-        // Filter
-        if ( ! flags || ( prFlags & flags ) )
-        {
-          // retrieve layer types if needed
-          if ( ! dontResolveType && ( !pr.geometryColName.isNull() &&
-                                      ( pr.types.value( 0, Qgis::WkbType::Unknown ) == Qgis::WkbType::Unknown ||
-                                        pr.srids.value( 0, std::numeric_limits<int>::min() ) == std::numeric_limits<int>::min() ) ) )
-          {
-            conn->retrieveLayerTypes( pr, useEstimatedMetadata, feedback );
-          }
-          QgsPostgresProviderConnection::TableProperty property;
-          property.setFlags( prFlags );
-          for ( int i = 0; i < std::min( pr.types.size(), pr.srids.size() ) ; i++ )
-          {
-            property.addGeometryColumnType( pr.types.at( i ), QgsCoordinateReferenceSystem::fromEpsgId( pr.srids.at( i ) ) );
-          }
-          property.setTableName( pr.tableName );
-          property.setSchema( pr.schemaName );
-          property.setGeometryColumn( pr.geometryColName );
-          // These are candidates, not actual PKs
-          // property.setPrimaryKeyColumns( pr.pkCols );
-          property.setGeometryColumnCount( static_cast<int>( pr.nSpCols ) );
-          property.setComment( pr.tableComment );
-
-          // Get PKs
-          if ( pr.relKind == Qgis::PostgresRelKind::View
-               || pr.relKind == Qgis::PostgresRelKind::MaterializedView
-               || pr.relKind == Qgis::PostgresRelKind::ForeignTable )
-          {
-            // Set the candidates
-            property.setPrimaryKeyColumns( pr.pkCols );
-          }
-          else  // Fetch and set the real pks
-          {
-            try
-            {
-              const QList<QVariantList> pks = executeSqlPrivate( QStringLiteral( R"(
-              WITH pkrelid AS (
-              SELECT indexrelid AS idxri FROM pg_index WHERE indrelid='%1.%2'::regclass AND (indisprimary OR indisunique)
-                ORDER BY CASE WHEN indisprimary THEN 1 ELSE 2 END LIMIT 1)
-              SELECT attname FROM pg_index,pg_attribute, pkrelid
-              WHERE indexrelid=pkrelid.idxri AND indrelid=attrelid AND pg_attribute.attnum=any(pg_index.indkey);
-             )" ).arg( QgsPostgresConn::quotedIdentifier( pr.schemaName ),
-                                                  QgsPostgresConn::quotedIdentifier( pr.tableName ) ), false );
-              QStringList pkNames;
-              for ( const QVariantList &pk : std::as_const( pks ) )
-              {
-                pkNames.push_back( pk.first().toString() );
-              }
-              property.setPrimaryKeyColumns( pkNames );
-            }
-            catch ( const QgsProviderConnectionException &ex )
-            {
-              QgsDebugError( QStringLiteral( "Error retrieving primary keys: %1" ).arg( ex.what() ) );
-            }
-          }
-
-          tables.push_back( property );
-        }
-      }
-    }
-    QgsPostgresConnPool::instance()->releaseConnection( conn );
+    throw QgsProviderConnectionException( QObject::tr( "Table '%1' was not found in schema '%2'" )
+                                          .arg( table, schema ) );
   }
-  if ( ! errCause.isEmpty() )
-  {
-    throw QgsProviderConnectionException( errCause );
-  }
-  return tables;
 }
 
 QStringList QgsPostgresProviderConnection::schemas( ) const
